/*
Modified by claybird(2011.2.22)

Original code:
7zMain.c - Test application for 7z Decoder
2010-10-28 : Igor Pavlov : Public domain 
*/

#include "lzma/7z.h"
#include "lzma/7zAlloc.h"
#include "lzma/7zCrc.h"
#include "lzma/7zFile.h"

static ISzAlloc g_Alloc = { SzAlloc, SzFree };

static int Buf_EnsureSize(CBuf *dest, size_t size)
{
	if (dest->size >= size)
		return 1;
	Buf_Free(dest, &g_Alloc);
	return Buf_Create(dest, size, &g_Alloc);
}

static SRes Utf16_To_Char(CBuf *buf, const UInt16 *s, int fileMode)
{
	int len = 0;
	for (len = 0; s[len] != '\0'; len++);

	{
		int size = len * 3 + 100;
		if (!Buf_EnsureSize(buf, size))
			return SZ_ERROR_MEM;
		{
			char defaultChar = '_';
			BOOL defUsed;
			int numChars = WideCharToMultiByte(fileMode ?
				(
#ifdef UNDER_CE
				CP_ACP
#else
				AreFileApisANSI() ? CP_ACP : CP_OEMCP
#endif
				) : CP_OEMCP,
				0, s, len, (char *)buf->data, size, &defaultChar, &defUsed);
			if (numChars == 0 || numChars >= size)
				return SZ_ERROR_FAIL;
			buf->data[numChars] = 0;
			return SZ_OK;
		}
	}
}

static WRes MyCreateDir(const UInt16 *name)
{
	return CreateDirectoryW(name, NULL) ? 0 : GetLastError();
}

static WRes OutFile_OpenUtf16(CSzFile *p, const UInt16 *name)
{
	return OutFile_OpenW(p, name);
}


//fBNg͖
int SevenZip_Extract(const char* aname, const char* dll,char* dll_rel_path)
{
	CFileInStream archiveStream;
	CLookToRead lookStream;
	CSzArEx db;
	SRes res;
	ISzAlloc allocImp;
	ISzAlloc allocTempImp;
	UInt16 *temp = NULL;
	size_t tempSize = 0;

	allocImp.Alloc = SzAlloc;
	allocImp.Free = SzFree;

	allocTempImp.Alloc = SzAllocTemp;
	allocTempImp.Free = SzFreeTemp;

	if (InFile_Open(&archiveStream.file, aname))
	{
		//PrintError("can not open input file");
		return 1;
	}

	FileInStream_CreateVTable(&archiveStream);
	LookToRead_CreateVTable(&lookStream, False);

	lookStream.realStream = &archiveStream.s;
	LookToRead_Init(&lookStream);

	CrcGenerateTable();

	SzArEx_Init(&db);
	res = SzArEx_Open(&db, &lookStream.s, &allocImp, &allocTempImp);
	if (res == SZ_OK)
	{
		UInt32 i;


		/*
		if you need cache, use these 3 variables.
		if you use external function, you can make these variable as static.
		*/
		UInt32 blockIndex = 0xFFFFFFFF; /* it can have any value before first call (if outBuffer = 0) */
		Byte *outBuffer = 0; /* it must be 0 before first call for each new archive. */
		size_t outBufferSize = 0;  /* it can have any value before first call (if outBuffer = 0) */

		for (i = 0; i < db.db.NumFiles; i++)
		{
			size_t offset = 0;
			size_t outSizeProcessed = 0;
			const CSzFileItem *f = db.db.Files + i;
			size_t len;
			if (f->IsDir)continue;
			len = SzArEx_GetFileNameUtf16(&db, i, NULL);

			if (len > tempSize)
			{
				SzFree(NULL, temp);
				tempSize = len;
				temp = (UInt16 *)SzAlloc(NULL, tempSize * sizeof(temp[0]));
				if (temp == 0)
				{
					res = SZ_ERROR_MEM;
					break;
				}
			}

			SzArEx_GetFileNameUtf16(&db, i, temp);
			if (!f->IsDir)
			{
				res = SzArEx_Extract(&db, &lookStream.s, i,
					&blockIndex, &outBuffer, &outBufferSize,
					&offset, &outSizeProcessed,
					&allocImp, &allocTempImp);
				if (res != SZ_OK)
					break;
			}
			{
				CSzFile outFile;
				size_t processedSize;
				size_t j;
				UInt16 *name = (UInt16 *)temp;
				const UInt16 *destPath = (const UInt16 *)name;
				for (j = 0; name[j] != 0; j++)
					if (name[j] == '/')
					{
						destPath = name + j + 1;
					}

					{
						CBuf buf;
						Buf_Init(&buf);
						res = Utf16_To_Char(&buf, destPath, 0);
						if (res == SZ_OK){
							if( 0==_strcmpi( (const char *)buf.data, dll ) ) // DLL̈ʒuL
								strcpy(dll_rel_path,(const char *)buf.data);
						}
						Buf_Free(&buf, &g_Alloc);
					}

					if (OutFile_OpenUtf16(&outFile, destPath)){
						//PrintError("can not open output file");
						res = SZ_ERROR_FAIL;
						break;
					}
					processedSize = outSizeProcessed;
					if (File_Write(&outFile, outBuffer + offset, &processedSize) != 0 || processedSize != outSizeProcessed)
					{
						//PrintError("can not write output file");
						res = SZ_ERROR_FAIL;
						break;
					}
					if (File_Close(&outFile)){
						//PrintError("can not close output file");
						res = SZ_ERROR_FAIL;
						break;
					}
					if (f->AttribDefined)
						SetFileAttributesW(destPath, f->Attrib);
			}
			//printf("\n");
		}
		IAlloc_Free(&allocImp, outBuffer);
	}
	SzArEx_Free(&db, &allocImp);
	SzFree(NULL, temp);

	File_Close(&archiveStream.file);
	if (res == SZ_OK)
	{
		//printf("\nEverything is Ok\n");
		return 0;
	}
	//if (res == SZ_ERROR_UNSUPPORTED)
	//  PrintError("decoder doesn't support this archive");
	//else if (res == SZ_ERROR_MEM)
	//  PrintError("can not allocate memory");
	//else if (res == SZ_ERROR_CRC)
	//  PrintError("CRC error");
	//else
	//  printf("\nERROR #%d\n", res);
	return 1;
}

