
#include "stdafx.h"
#include "kiutil.h"

///////////////

void kiutil::timeSet( const char* fname, FILETIME* pft )
{
	HANDLE han = CreateFile( fname,
							 GENERIC_READ | GENERIC_WRITE,
							 FILE_SHARE_READ,NULL,
							 OPEN_EXISTING,
							 FILE_ATTRIBUTE_NORMAL,
							 NULL );
	if( han==INVALID_HANDLE_VALUE )
		return;

	SetFileTime( han,pft,NULL,pft );

	CloseHandle( han );
}

void kiutil::timeSet( const char* fname, DWORD sec )
{
	struct tm time;
	if( gmtime_s(&time,(time_t*)&sec)!=NULL )
	{
		FILETIME ft;
		SYSTEMTIME sys;

		sys.wYear =      time.tm_year+1900;
		sys.wMonth =     time.tm_mon+1;
		sys.wDayOfWeek = time.tm_wday;
		sys.wDay =       time.tm_mday;
		sys.wHour =      time.tm_hour;
		sys.wMinute =    time.tm_min;
		sys.wSecond =    time.tm_sec;
		sys.wMilliseconds = 0;
		SystemTimeToFileTime(&sys,&ft);
		timeSet( fname,&ft );
	}
}

void kiutil::timeSet( const char* fname,WORD date,WORD time )
{
	FILETIME ft,lc;
	if( DosDateTimeToFileTime( date, time, &lc ) )
	{
		if( LocalFileTimeToFileTime( &lc, &ft ) )
			timeSet( fname,&ft );
	}
}

//////////////

void kiutil::wndFront( HWND wnd )
{
	static DWORD Ver = GetVersion();

	if( ( (Ver&0x80000000) && LOBYTE(LOWORD(Ver))>=4 && HIBYTE(LOWORD(Ver))>=10 ) ||
	    (!(Ver&0x80000000) && LOBYTE(LOWORD(Ver))>=5 )) // VWindows
	{
		DWORD pid;
		DWORD thread1 = GetWindowThreadProcessId( GetForegroundWindow(),&pid );
		DWORD thread2 = GetCurrentThreadId();
		AttachThreadInput( thread2, thread1, TRUE );
		SetForegroundWindow( wnd );
		AttachThreadInput( thread2, thread1, FALSE );
		BringWindowToTop( wnd );
	}
	else  // ÂWindows
		SetForegroundWindow( wnd );
}

///////////////

char kiutil::lb[256];

void kiutil::pathInit()
{
	lb[0] = 0;
	for( int c=1; c!=256; c++ )
		lb[c] = (IsDBCSLeadByte(c) ? 2 : 1);
}

#define isdblb(c) (lb[(unsigned char)(c)]==2)
#define step(p) (p+=lb[(unsigned char)*(p)])

char* kiutil::pathMake( char* path )
{
	char* st = path;

	while( *st=='/' || *st=='\\' || *st=='?' )
		st++;
	if( st[0]!='\0' && st[1]==':' )
		st+=2;
	while( *st=='/' || *st=='\\' || *st=='?' )
		st++;

	for( unsigned char *p=(unsigned char*)st; *p!='\0'; step(p) )
	{
		if( isdblb(*p) )
			continue;

		if( *p=='\\' || *p=='/' )
		{
			*p='\0';
			CreateDirectory( st, NULL );
			*p='\\';
		}
		else if( *p<' ' || ( *p>'~' && !( 0xa0<=*p && *p<=0xdf ) ) || strchr(":*?\"<>|",*p) )
			*p = '_';
	}

	return st;
}

void kiutil::pathMakeAbs( char* path )
{
	int i=0;
	for( char* p=path; *p!='\0'; step(p) )
	{
		if( i++ < 4 ) // ŏ̂Sȓ \ ̓hCu\AƂƂɂĂB
			continue;

		if( *p=='\\' )
		{
			*p='\0';
			CreateDirectory( path, NULL );
			*p='\\';
		}
	}
}

void kiutil::pathSplit( const char* path, int* y, int* d )
{
	*y=-1, *d=-1;
	for( const char* x=path; *x!='\0'; step(x) ) 
	{
		if( *x=='\\' || *x=='/' )	*y=x-path,*d=-1;
		else if( *x=='.' )			*d=x-path;
	}
}

const char* kiutil::pathExt( const char* path )
{
	int y,d;
	kiutil::pathSplit( path,&y,&d );
	return (d!=-1) ? path+d+1 : path+strlen(path);
}

const char* kiutil::pathName( const char* path )
{
	int y,d;
	kiutil::pathSplit( path,&y,&d );
	return path+y+1;
}

#undef step
#undef isdblb

//////////////

char* kiutil::getline( char* str, int size, FILE* fp )
{
	if( size>0 )
	{
		char* p=str;

		while( --size )
		{
			int c = fgetc(fp);

			if( c<=0 ) // EOF || '\0'
				break;
			else if( c=='\n' )
				break;
			else if( c=='\r' )
			{
				c = fgetc(fp);
				if( c!=EOF && c!='\n' )
					ungetc( c,fp );
				break;
			}
			else
				*(p++) = c;
		}

		*p='\0';
	}

	return feof(fp) ? NULL : str;
}

/*void kiutil::getOriginalName( char* nw, const char* od, char* ext )
{
	int y,d;
	kiutil::pathSplit( od,&y,&d );
	strcpy( nw,od+y+1 );	//ZLeBD܂Ȃ

	if( ext )
	{
		strcat( nw,"." );
		strcat( nw,ext );
	}
	else
		nw[d-y-1]='\0';
}
*/

//http://www.s34.co.jp/cpptechdoc/article/art_24_01/
template<class E, class T, class A>
std::basic_string<E,T,A>
replace_all(
  const std::basic_string<E,T,A>& source,
  const std::basic_string<E,T,A>& pattern,
  const std::basic_string<E,T,A>& placement
  ) {
  std::basic_string<E,T,A> result;
  std::basic_string<E,T,A>::size_type pos_before = 0;
  std::basic_string<E,T,A>::size_type pos = 0;
  std::basic_string<E,T,A>::size_type len = pattern.size();
  while( ( pos = source.find( pattern, pos ) ) != std::string::npos ) {
    result.append(source, pos_before, pos - pos_before);
    result.append(placement);
    pos += len ;
    pos_before = pos ;
  }
  result.append(source, pos_before, source.size() - pos_before) ;
  return result;
}

//̃vÕt@CԂ
LPCTSTR UtilGetModulePath()
{
	static TCHAR s_szExePath[_MAX_PATH+1]={0};
	if(s_szExePath[0]!='\0')return s_szExePath;

	GetModuleFileName(GetModuleHandle(NULL), s_szExePath, _MAX_PATH);	//{̂̃pX擾
	return s_szExePath;
}

//̃vÔĂfBNg̃pXԂ
LPCTSTR UtilGetModuleDirectoryPath()
{
	static TCHAR s_szDirPath[_MAX_PATH+1]={0};
	if(s_szDirPath[0]!='\0')return s_szDirPath;

	GetModuleFileName(GetModuleHandle(NULL), s_szDirPath, _MAX_PATH);	//{̂̃pX擾
	PathRemoveFileSpec(s_szDirPath);
	return s_szDirPath;
}

//ϐQƂA`Ŏ擾
void UtilGetEnvInfo(std::map<std::string,std::string> &envInfo)
{
	LPTSTR lpEnvStr = GetEnvironmentStrings();
	LPCTSTR lpStr    = lpEnvStr;

	for(;*lpStr!='\0';){
		LPCTSTR lpSplit=lpStr+1;	//ϐ̐擪'='ꍇɔ
		for(;*lpSplit!='\0' && *lpSplit != '=';lpSplit++)continue;

		//---ϐ
		std::string strKey(lpStr,lpSplit);
		//啶ɐK
		std::transform(strKey.begin(), strKey.end(), strKey.begin(), std::toupper);
		//---ϐ̒l
		std::string strValue=lpSplit+1;
		envInfo[strKey]=strValue;

		for(lpStr=lpSplit;*lpStr!='\0';lpStr++)continue;
		lpStr++;
	}

	FreeEnvironmentStrings(lpEnvStr);
}

//UtilExpandTemplateString()̃p[^WJɕKvȏ\z
void UtilMakeExpandInformation(std::map<std::string,std::string> &envInfo)
{
	//ϐō\z
	std::map<std::string,std::string> envs;
	UtilGetEnvInfo(envs);
	for(std::map<std::string,std::string>::iterator ite=envs.begin();ite!=envs.end();++ite){
		//%ENVIRONMENT%̌`ɕϊ
		envInfo['%'+(*ite).first+'%']=(*ite).second.c_str();
	}

	//---LhaForge{̂̏
	envInfo["ProgramPath"]=UtilGetModulePath();
	envInfo["ProgramFileName"]=PathFindFileName(UtilGetModulePath());

	char buffer[_MAX_PATH];
	strncpy_s(buffer,_MAX_PATH,UtilGetModuleDirectoryPath(),_MAX_PATH);
	PathRemoveBackslash(buffer);
	envInfo["ProgramDir"]=buffer;


	PathStripToRoot(buffer);
	//Backslash菜;RemoveBackslashł̓hCuBackslash菜Ȃ
	PathRemoveBackslash(buffer);
	envInfo["ProgramDrive"]=buffer;
}


//w肳ꂽtH[}bgŏꂽWJ
void UtilExpandTemplateString(std::string &strOut,LPCTSTR lpszFormat,const std::map<std::string,std::string> &env)
{
	strOut=lpszFormat;
	for(std::map<std::string,std::string>::const_iterator ite=env.begin();ite!=env.end();++ite){
		//%Ŏn܂ubN͊ϐƌȂ̂{}ɂC͍sȂ
		std::string strKey=( (*ite).first[0]=='%' ? (*ite).first : "{"+(*ite).first+"}" );
		strOut=replace_all(strOut,strKey,(*ite).second);
	}
}

//tpX΃pX̎擾
bool UtilGetCompletePathName(std::string &_FullPath,LPCTSTR lpszFileName)
{
	if(!lpszFileName||_tcslen(lpszFileName)<=0){
		//TRACE(_T("t@Cw肳ĂȂ\n"));
		return false;
	}

	//---΃pX擾
	TCHAR szAbsPath[_MAX_PATH+1]={0};
	{
		TCHAR Buffer[_MAX_PATH+1]={0};	//[gǂ̃`FbNs
		_tcsncpy_s(Buffer,lpszFileName,_MAX_PATH);
		PathAddBackslash(Buffer);
		if(PathIsRoot(Buffer)){
			//hCuw肳ĂꍇA
			//_tfullpath͂̃hCũJgfBNg擾Ă܂
			_tcsncpy_s(szAbsPath,lpszFileName,_MAX_PATH);
		}
		else if(!_tfullpath(szAbsPath,lpszFileName,_MAX_PATH)){
			//TRACE(_T("΃pX擾s\n"));
			return false;
		}
	}

	if(!PathFileExists(szAbsPath)&&!PathIsDirectory(szAbsPath)){
		//pXt@C̓fBNgƂđ݂ȂȂAG[Ƃ
		//TRACE(_T("t@C݂Ȃ\n"));
		return false;
	}
	if(!GetLongPathName(szAbsPath,szAbsPath,_MAX_PATH)){
		//TRACE(_T("Ot@C擾s\n"));
		return false;
	}
	_FullPath=szAbsPath;
	return true;
}
